import { Type, ViewContainerRef, EmbeddedViewRef, TemplateRef } from '@angular/core';
import { RenderingField } from './rendering-field';
import { GenericFieldValue } from '@sitecore-jss/sitecore-jss/layout';
import { MetadataKind } from '@sitecore-jss/sitecore-jss/editing';
import * as i0 from "@angular/core";
/**
 * Base class that contains common functionality for the field directives.
 */
export declare abstract class BaseFieldDirective {
    protected viewContainer: ViewContainerRef;
    protected viewRef: EmbeddedViewRef<unknown>;
    protected abstract field: RenderingField<GenericFieldValue>;
    protected abstract editable: boolean;
    /**
     * Custom template to render in Pages in Metadata edit mode if field value is empty
     */
    protected abstract emptyFieldEditingTemplate: TemplateRef<unknown>;
    /**
     * Default component to render in Pages in Metadata edit mode if field value is empty and emptyFieldEditingTemplate is not provided
     */
    protected abstract defaultFieldEditingComponent: Type<unknown>;
    constructor(viewContainer: ViewContainerRef);
    /**
     * Determines if directive should render the field as is
     * Returns true if we are in edit mode 'chromes' (field.editable is present) or field is not empty
     */
    protected shouldRender(): boolean;
    /**
     * Renders the empty field markup which is required by Pages in editMode 'metadata' in case field is empty.
     */
    protected renderEmpty(): void;
    /**
     * Renders a metadata chrome marker for the field. Required by Pages in editMode 'metadata'.
     * @param {string} kind - 'open' or 'close' to indicate the start or end of the metadata chrome
     */
    protected renderMetadata(kind: MetadataKind): void;
    static ɵfac: i0.ɵɵFactoryDeclaration<BaseFieldDirective, never>;
    static ɵdir: i0.ɵɵDirectiveDeclaration<BaseFieldDirective, never, never, {}, {}, never, never, false, never>;
}
