import { ComponentRendering } from '@sitecore-jss/sitecore-jss/layout';
import { OnInit, ElementRef, OnDestroy } from '@angular/core';
import { EdgeConfigToken } from '../services/shared.token';
import { JssStateService } from '../services/jss-state.service';
import * as i0 from "@angular/core";
/**
 * Mock function to replace the form module functions for `testing` purposes.
 * @param {any} formModule - The form module to mock
 */
export declare const mockFormModule: (formModule: any) => void;
/**
 * Shape of the Form component rendering data.
 * FormId is the rendering parameter that specifies the ID of the Sitecore Form to render.
 */
export type FormRendering = {
    params: {
        FormId: string;
        styles?: string;
    };
} & ComponentRendering;
/**
 * A component that renders a Sitecore Form.
 * It fetches the form markup from the Sitecore Edge service and renders it in the component's template.
 */
export declare class FormComponent implements OnInit, OnDestroy {
    private edgeConfig;
    private platformId;
    private elRef;
    private jssState;
    /**
     * The rendering data for the component
     */
    rendering: FormRendering;
    hasError: boolean;
    isEditing: boolean;
    private contextSubscription;
    constructor(edgeConfig: EdgeConfigToken, platformId: {
        [key: string]: unknown;
    }, elRef: ElementRef<HTMLElement>, jssState: JssStateService);
    ngOnInit(): void;
    ngOnDestroy(): void;
    /**
     * Fetches the form markup from the Sitecore Edge service and renders it in the component's template.
     */
    loadForm(): Promise<void>;
    static ɵfac: i0.ɵɵFactoryDeclaration<FormComponent, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<FormComponent, "app-form", never, { "rendering": { "alias": "rendering"; "required": false; }; }, {}, never, never, false, never>;
}
