import { ChangeDetectorRef, DoCheck, ElementRef, EventEmitter, KeyValueDiffers, OnChanges, OnDestroy, OnInit, Renderer2, SimpleChanges, Type } from '@angular/core';
import { Data, Router } from '@angular/router';
import { ComponentRendering, HtmlElementRendering } from '@sitecore-jss/sitecore-jss/layout';
import { ComponentFactoryResult, JssComponentFactoryService } from '../services/jss-component-factory.service';
import { DataResolver, GuardResolver } from '../services/placeholder.token';
import { PlaceholderLoadingDirective } from './placeholder-loading.directive';
import { RenderEachDirective } from './render-each.directive';
import { RenderEmptyDirective } from './render-empty.directive';
import { JssStateService } from '../services/jss-state.service';
import * as i0 from "@angular/core";
export interface FactoryWithData {
    factory: ComponentFactoryResult;
    data?: Data;
}
export declare class PlaceholderComponent implements OnInit, OnChanges, DoCheck, OnDestroy {
    private differs;
    private componentFactory;
    private changeDetectorRef;
    private elementRef;
    private renderer;
    private router;
    private missingComponentComponent;
    private hiddenRenderingComponent;
    private guardResolver;
    private dataResolver;
    private platformId;
    private jssState;
    name?: string;
    rendering: ComponentRendering;
    renderings?: Array<ComponentRendering | HtmlElementRendering>;
    outputs: {
        [k: string]: (eventType: unknown) => void;
    };
    clientOnly: boolean;
    loaded: EventEmitter<string>;
    failed: EventEmitter<Error>;
    renderEachTemplate: RenderEachDirective;
    renderEmptyTemplate: RenderEmptyDirective;
    placeholderLoading?: PlaceholderLoadingDirective;
    private view;
    private metadataNode;
    isLoading: boolean;
    metadataMode: boolean;
    chromeType: string;
    private _inputs;
    private _differ;
    private _componentInstances;
    private placeholderData?;
    private destroyed;
    private parentStyleAttribute;
    private contextSubscription;
    constructor(differs: KeyValueDiffers, componentFactory: JssComponentFactoryService, changeDetectorRef: ChangeDetectorRef, elementRef: ElementRef, renderer: Renderer2, router: Router, missingComponentComponent: Type<unknown>, hiddenRenderingComponent: Type<unknown>, guardResolver: GuardResolver, dataResolver: DataResolver, platformId: object, jssState: JssStateService);
    set inputs(value: {
        [key: string]: unknown;
    });
    ngOnInit(): void;
    ngOnDestroy(): void;
    ngOnChanges(changes: SimpleChanges): void;
    ngDoCheck(): void;
    /**
     * Gets id for Metadata code blocks, in specific format
     * Metadata code blocks will wrap be added around placeholder content and each rendering component
     * to allow for editing integration in Pages.
     * @param {string} kind code block type ("open" or "close"). "open" is added before an element, and "close" added after one.
     * @param {string?} renderingId rendering uid to apply as id to code block
     * @returns {string} formatted id value for code HTML node
     */
    getCodeBlockId: (kind: string, renderingId?: string) => string | undefined;
    /**
     * Get renderings/components to be rendered for current placeholder name
     * Can modify the inner placeholders collection to adjust to using SXA dynamic placeholders
     * @returns {ComponentRendering<ComponentFields> | HtmlElementRendering[] | null} List of renderings to be rendered
     */
    private getPlaceholder;
    private _setComponentInputs;
    private _subscribeComponentOutputs;
    private _render;
    private _renderTemplatedComponent;
    private _renderEmbeddedComponent;
    static ɵfac: i0.ɵɵFactoryDeclaration<PlaceholderComponent, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<PlaceholderComponent, "sc-placeholder,[sc-placeholder]", never, { "name": { "alias": "name"; "required": false; }; "rendering": { "alias": "rendering"; "required": false; }; "renderings": { "alias": "renderings"; "required": false; }; "outputs": { "alias": "outputs"; "required": false; }; "clientOnly": { "alias": "clientOnly"; "required": false; }; "inputs": { "alias": "inputs"; "required": false; }; }, { "loaded": "loaded"; "failed": "failed"; }, ["renderEachTemplate", "renderEmptyTemplate", "placeholderLoading"], never, false, never>;
}
