import * as i0 from '@angular/core';
import { Input, Directive, HostBinding, Component, InjectionToken, createNgModule, Inject, Injectable, EventEmitter, PLATFORM_ID, ViewContainerRef, TemplateRef, ViewChild, ContentChild, Output, ɵisPromise as _isPromise, Injector, NgModule } from '@angular/core';
import { mediaApi } from '@sitecore-jss/sitecore-jss/media';
export { mediaApi } from '@sitecore-jss/sitecore-jss/media';
import { isFieldValueEmpty, EditMode, isDynamicPlaceholder, getDynamicPlaceholderPattern, LayoutServicePageState } from '@sitecore-jss/sitecore-jss/layout';
export { EditMode, GraphQLLayoutService, LayoutServicePageState, RestLayoutService, getChildPlaceholder, getContentStylesheetLink, getFieldValue } from '@sitecore-jss/sitecore-jss/layout';
import { MetadataKind, DEFAULT_PLACEHOLDER_UID, mapButtonToCommand, getJssPagesClientData } from '@sitecore-jss/sitecore-jss/editing';
export { DefaultEditFrameButton, DefaultEditFrameButtons, handleEditorAnchors, isEditorActive, resetEditorChromes } from '@sitecore-jss/sitecore-jss/editing';
import * as i1 from '@angular/router';
import { RedirectCommand, UrlTree, ActivatedRoute, Router, ROUTES } from '@angular/router';
import { isAbsoluteUrl, isServer } from '@sitecore-jss/sitecore-jss/utils';
export { isServer } from '@sitecore-jss/sitecore-jss/utils';
import * as i1$1 from '@angular/common';
import { isPlatformServer, DOCUMENT, isPlatformBrowser, DatePipe, CommonModule } from '@angular/common';
import { BehaviorSubject, Observable, isObservable, from, of, lastValueFrom } from 'rxjs';
import { takeWhile, take, mergeMap } from 'rxjs/operators';
import { constants, form, debug } from '@sitecore-jss/sitecore-jss';
export { ClientError, MemoryCacheClient, NativeDataFetcher, constants, debug, enableDebug } from '@sitecore-jss/sitecore-jss';
export { GraphQLDictionaryService, RestDictionaryService } from '@sitecore-jss/sitecore-jss/i18n';
export { DefaultRetryStrategy, GraphQLRequestClient, getEdgeProxyContentUrl } from '@sitecore-jss/sitecore-jss/graphql';
export { CdpHelper, DEFAULT_VARIANT, GraphQLPersonalizeService, getGroomedVariantIds, personalizeLayout } from '@sitecore-jss/sitecore-jss/personalize';
export { trackingApi } from '@sitecore-jss/sitecore-jss/tracking';

/**
 * File fields cannot be managed via the EE. We never output "editable."
 */
class FileDirective {
    viewContainer;
    templateRef;
    field;
    viewRef;
    constructor(viewContainer, templateRef) {
        this.viewContainer = viewContainer;
        this.templateRef = templateRef;
    }
    ngOnChanges(changes) {
        if (changes.field) {
            if (!this.viewRef) {
                this.viewContainer.clear();
                this.viewRef = this.viewContainer.createEmbeddedView(this.templateRef);
            }
            this.updateView();
        }
    }
    updateView() {
        const field = this.field;
        if (!field || (!field.value && !field.src)) {
            return;
        }
        const file = field.src ? field : field.value;
        this.viewRef.rootNodes.forEach((node) => {
            if (!file)
                return;
            node.href = file.src;
            if (node.innerHTML === '') {
                node.innerHTML = file.title || file.displayName;
            }
        });
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: FileDirective, deps: [{ token: i0.ViewContainerRef }, { token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.13", type: FileDirective, selector: "[scFile]", inputs: { field: ["scFile", "field"] }, usesOnChanges: true, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: FileDirective, decorators: [{
            type: Directive,
            args: [{ selector: '[scFile]' }]
        }], ctorParameters: () => [{ type: i0.ViewContainerRef }, { type: i0.TemplateRef }], propDecorators: { field: [{
                type: Input,
                args: ['scFile']
            }] } });

/**
 * Component that renders a field' metadata chrome element.
 */
class FieldMetadataMarkerComponent {
    metadata;
    kind = MetadataKind.Open;
    get metadataString() {
        return this.metadata ? JSON.stringify(this.metadata) : '';
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: FieldMetadataMarkerComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: FieldMetadataMarkerComponent, selector: "code[scFieldMetadataMarker]", inputs: { metadata: "metadata", kind: "kind" }, host: { properties: { "attr.type": "\"text/sitecore\"", "attr.chrometype": "\"field\"", "class": "\"scpm\"", "attr.kind": "this.kind" } }, ngImport: i0, template: '{{ metadataString }}', isInline: true });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: FieldMetadataMarkerComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'code[scFieldMetadataMarker]',
                    template: '{{ metadataString }}',
                    // eslint-disable-next-line @angular-eslint/no-host-metadata-property -- the only way to set static attributes
                    host: {
                        '[attr.type]': '"text/sitecore"',
                        '[attr.chrometype]': '"field"',
                        '[class]': '"scpm"',
                    },
                }]
        }], propDecorators: { metadata: [{
                type: Input
            }], kind: [{
                type: HostBinding,
                args: ['attr.kind']
            }, {
                type: Input
            }] } });

/**
 * Base class that contains common functionality for the field directives.
 */
class BaseFieldDirective {
    viewContainer;
    viewRef;
    constructor(viewContainer) {
        this.viewContainer = viewContainer;
    }
    /**
     * Determines if directive should render the field as is
     * Returns true if we are in edit mode 'chromes' (field.editable is present) or field is not empty
     */
    shouldRender() {
        return !!this.field?.editable || !isFieldValueEmpty(this.field);
    }
    /**
     * Renders the empty field markup which is required by Pages in editMode 'metadata' in case field is empty.
     */
    renderEmpty() {
        if (this.field?.metadata && this.editable) {
            this.renderMetadata(MetadataKind.Open);
            if (this.emptyFieldEditingTemplate) {
                this.viewContainer.createEmbeddedView(this.emptyFieldEditingTemplate);
            }
            else {
                this.viewContainer.createComponent(this.defaultFieldEditingComponent);
            }
            this.renderMetadata(MetadataKind.Close);
        }
    }
    /**
     * Renders a metadata chrome marker for the field. Required by Pages in editMode 'metadata'.
     * @param {string} kind - 'open' or 'close' to indicate the start or end of the metadata chrome
     */
    renderMetadata(kind) {
        if (this.field?.metadata && this.editable) {
            const metadataChrome = this.viewContainer.createComponent(FieldMetadataMarkerComponent);
            metadataChrome.setInput('kind', kind);
            if (kind === MetadataKind.Open) {
                metadataChrome.setInput('metadata', this.field.metadata);
            }
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: BaseFieldDirective, deps: [{ token: i0.ViewContainerRef }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.13", type: BaseFieldDirective, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: BaseFieldDirective, decorators: [{
            type: Directive
        }], ctorParameters: () => [{ type: i0.ViewContainerRef }] });

/**
 * Default component that will be rendered in pages when image field is empty.
 */
class DefaultEmptyImageFieldEditingComponent {
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DefaultEmptyImageFieldEditingComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: DefaultEmptyImageFieldEditingComponent, selector: "[sc-default-empty-image-field-editing-placeholder]", ngImport: i0, template: `
    <img
      alt=""
      src='data:image/svg+xml,%3Csvg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 240 240" style="enable-background:new 0 0 240 240;" xml:space="preserve"%3E%3Cstyle type="text/css"%3E .st0%7Bfill:none;%7D .st1%7Bfill:%23969696;%7D .st2%7Bfill:%23FFFFFF;%7D .st3%7Bfill:%23FFFFFF;stroke:%23FFFFFF;stroke-width:0.75;stroke-miterlimit:10;%7D%0A%3C/style%3E%3Cg%3E%3Crect class="st0" width="240" height="240"/%3E%3Cg%3E%3Cg%3E%3Crect x="20" y="20" class="st1" width="200" height="200"/%3E%3C/g%3E%3Cg%3E%3Ccircle class="st2" cx="174" cy="67" r="14"/%3E%3Cpath class="st2" d="M174,54c7.17,0,13,5.83,13,13s-5.83,13-13,13s-13-5.83-13-13S166.83,54,174,54 M174,52 c-8.28,0-15,6.72-15,15s6.72,15,15,15s15-6.72,15-15S182.28,52,174,52L174,52z"/%3E%3C/g%3E%3Cpolyline class="st3" points="29.5,179.25 81.32,122.25 95.41,137.75 137.23,91.75 209.5,179.75 "/%3E%3C/g%3E%3C/g%3E%3C/svg%3E'
      class="scEmptyImage"
    />
  `, isInline: true, styles: [":host{min-width:48px;min-height:48px;max-width:400px;max-height:400px;cursor:pointer}\n"] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DefaultEmptyImageFieldEditingComponent, decorators: [{
            type: Component,
            args: [{ selector: '[sc-default-empty-image-field-editing-placeholder]', template: `
    <img
      alt=""
      src='data:image/svg+xml,%3Csvg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 240 240" style="enable-background:new 0 0 240 240;" xml:space="preserve"%3E%3Cstyle type="text/css"%3E .st0%7Bfill:none;%7D .st1%7Bfill:%23969696;%7D .st2%7Bfill:%23FFFFFF;%7D .st3%7Bfill:%23FFFFFF;stroke:%23FFFFFF;stroke-width:0.75;stroke-miterlimit:10;%7D%0A%3C/style%3E%3Cg%3E%3Crect class="st0" width="240" height="240"/%3E%3Cg%3E%3Cg%3E%3Crect x="20" y="20" class="st1" width="200" height="200"/%3E%3C/g%3E%3Cg%3E%3Ccircle class="st2" cx="174" cy="67" r="14"/%3E%3Cpath class="st2" d="M174,54c7.17,0,13,5.83,13,13s-5.83,13-13,13s-13-5.83-13-13S166.83,54,174,54 M174,52 c-8.28,0-15,6.72-15,15s6.72,15,15,15s15-6.72,15-15S182.28,52,174,52L174,52z"/%3E%3C/g%3E%3Cpolyline class="st3" points="29.5,179.25 81.32,122.25 95.41,137.75 137.23,91.75 209.5,179.75 "/%3E%3C/g%3E%3C/g%3E%3C/svg%3E'
      class="scEmptyImage"
    />
  `, styles: [":host{min-width:48px;min-height:48px;max-width:400px;max-height:400px;cursor:pointer}\n"] }]
        }] });

class ImageDirective extends BaseFieldDirective {
    templateRef;
    renderer;
    elementRef;
    field;
    editable = true;
    /**
     * Custom regexp that finds media URL prefix that will be replaced by `/-/jssmedia` or `/~/jssmedia`.
     * @example
     * /\/([-~]{1})assets\//i
     * /-assets/website -> /-/jssmedia/website
     * /~assets/website -> /~/jssmedia/website
     */
    mediaUrlPrefix;
    urlParams = {};
    attrs = {};
    /**
     * Custom template to render in Pages in Metadata edit mode if field value is empty
     */
    emptyFieldEditingTemplate;
    /**
     * Default component to render in Pages in Metadata edit mode if field value is empty and emptyFieldEditingTemplate is not provided
     */
    defaultFieldEditingComponent;
    inlineRef = null;
    constructor(viewContainer, templateRef, renderer, elementRef) {
        super(viewContainer);
        this.templateRef = templateRef;
        this.renderer = renderer;
        this.elementRef = elementRef;
        this.defaultFieldEditingComponent = DefaultEmptyImageFieldEditingComponent;
    }
    ngOnChanges(changes) {
        if (changes.field || changes.editable || changes.urlParams || changes.attrs) {
            this.viewContainer.clear();
            if (this.inlineRef) {
                this.inlineRef.remove();
                this.inlineRef = null;
            }
            this.updateView();
        }
    }
    updateView() {
        if (!this.shouldRender()) {
            super.renderEmpty();
            return;
        }
        const overrideAttrs = {
            ...this.getElementAttrs(),
            ...this.attrs,
        };
        const media = this.field;
        let attrs = {};
        // we likely have an experience editor value, should be a string
        if (this.editable && media.editable) {
            const foundImg = mediaApi.findEditorImageTag(media.editable);
            if (!foundImg) {
                return this.renderInlineWrapper(media.editable);
            }
            attrs = this.getImageAttrs(foundImg.attrs, overrideAttrs, this.urlParams);
            if (!attrs) {
                return this.renderInlineWrapper(media.editable);
            }
            const tempImg = this.renderer.createElement('img');
            Object.entries(attrs).forEach(([key, attrValue]) => tempImg.setAttribute(key, attrValue));
            const editableMarkup = media.editable.replace(foundImg.imgTag, tempImg.outerHTML);
            return this.renderInlineWrapper(editableMarkup);
        }
        // some wise-guy/gal is passing in a 'raw' image object value
        const img = media.src ? media : media.value;
        if (!img) {
            return null;
        }
        attrs = this.getImageAttrs(img, overrideAttrs, this.urlParams);
        if (attrs) {
            this.renderMetadata(MetadataKind.Open);
            this.renderTemplate(attrs);
            this.renderMetadata(MetadataKind.Close);
        }
    }
    getImageAttrs(fieldAttrs, parsedAttrs, imageParams) {
        const combinedAttrs = {
            ...fieldAttrs,
            ...parsedAttrs,
        };
        // eslint-disable-next-line prefer-const
        let { src, srcSet, style, ...otherAttrs } = combinedAttrs;
        if (!src) {
            return null;
        }
        const newAttrs = {
            ...otherAttrs,
        };
        // update image URL for jss handler and image rendering params
        src = mediaApi.updateImageUrl(src, imageParams, this.mediaUrlPrefix);
        if (srcSet) {
            // replace with HTML-formatted srcset, including updated image URLs
            newAttrs.srcSet = mediaApi.getSrcSet(src, srcSet, imageParams, this.mediaUrlPrefix);
        }
        else {
            newAttrs.src = src;
        }
        if (style) {
            newAttrs.style = Object.entries(style)
                .map(([propName, value]) => `${propName}:${value}`)
                .join(';');
        }
        return newAttrs;
    }
    renderTemplate(imageProps) {
        const viewRef = this.viewContainer.createEmbeddedView(this.templateRef);
        viewRef.rootNodes.forEach((node) => {
            Object.entries(imageProps).forEach(([key, imgPropVal]) => this.renderer.setAttribute(node, key, imgPropVal));
        });
    }
    getElementAttrs() {
        const view = this.templateRef.createEmbeddedView(null);
        const element = view.rootNodes[0];
        if (!element) {
            view.destroy();
            return {};
        }
        const attrs = {};
        for (let i = 0; i < element.attributes.length; i++) {
            const attr = element.attributes.item(i);
            if (attr) {
                attrs[attr.name] = attr.value;
            }
        }
        view.destroy();
        return attrs;
    }
    renderInlineWrapper(editable) {
        const span = this.renderer.createElement('span');
        span.className = 'sc-image-wrapper';
        span.innerHTML = editable;
        const parentNode = this.renderer.parentNode(this.elementRef.nativeElement);
        this.renderer.insertBefore(parentNode, span, this.elementRef.nativeElement);
        parentNode.removeChild(this.elementRef.nativeElement);
        this.inlineRef = span;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ImageDirective, deps: [{ token: i0.ViewContainerRef }, { token: i0.TemplateRef }, { token: i0.Renderer2 }, { token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.13", type: ImageDirective, selector: "[scImage]", inputs: { field: ["scImage", "field"], editable: ["scImageEditable", "editable"], mediaUrlPrefix: ["scImageMediaUrlPrefix", "mediaUrlPrefix"], urlParams: ["scImageUrlParams", "urlParams"], attrs: ["scImageAttrs", "attrs"], emptyFieldEditingTemplate: ["scImageEmptyFieldEditingTemplate", "emptyFieldEditingTemplate"] }, usesInheritance: true, usesOnChanges: true, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ImageDirective, decorators: [{
            type: Directive,
            args: [{ selector: '[scImage]' }]
        }], ctorParameters: () => [{ type: i0.ViewContainerRef }, { type: i0.TemplateRef }, { type: i0.Renderer2 }, { type: i0.ElementRef }], propDecorators: { field: [{
                type: Input,
                args: ['scImage']
            }], editable: [{
                type: Input,
                args: ['scImageEditable']
            }], mediaUrlPrefix: [{
                type: Input,
                args: ['scImageMediaUrlPrefix']
            }], urlParams: [{
                type: Input,
                args: ['scImageUrlParams']
            }], attrs: [{
                type: Input,
                args: ['scImageAttrs']
            }], emptyFieldEditingTemplate: [{
                type: Input,
                args: ['scImageEmptyFieldEditingTemplate']
            }] } });

/**
 * Default component that will be rendered in pages when field is empty; applies for text, richtext, date and link fields.
 */
class DefaultEmptyFieldEditingComponent {
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DefaultEmptyFieldEditingComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: DefaultEmptyFieldEditingComponent, selector: "<span>[sc-default-empty-text-field-editing-placeholder]</span>", ngImport: i0, template: '[No text in field]', isInline: true });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DefaultEmptyFieldEditingComponent, decorators: [{
            type: Component,
            args: [{
                    selector: '<span>[sc-default-empty-text-field-editing-placeholder]</span>',
                    template: '[No text in field]',
                }]
        }] });

class LinkDirective extends BaseFieldDirective {
    templateRef;
    renderer;
    elementRef;
    editable = true;
    attrs = {};
    field;
    /**
     * Custom template to render in Pages in Metadata edit mode if field value is empty
     */
    emptyFieldEditingTemplate;
    /**
     * Default component to render in Pages in Metadata edit mode if field value is empty and emptyFieldEditingTemplate is not provided
     */
    defaultFieldEditingComponent;
    inlineRef = null;
    constructor(viewContainer, templateRef, renderer, elementRef) {
        super(viewContainer);
        this.templateRef = templateRef;
        this.renderer = renderer;
        this.elementRef = elementRef;
        this.defaultFieldEditingComponent = DefaultEmptyFieldEditingComponent;
    }
    ngOnChanges(changes) {
        if (changes.field || changes.editable || changes.attrs) {
            this.viewContainer.clear();
            if (this.inlineRef) {
                this.inlineRef.remove();
                this.inlineRef = null;
            }
            this.updateView();
        }
    }
    renderTemplate(props, linkText) {
        const viewRef = this.viewContainer.createEmbeddedView(this.templateRef);
        viewRef.rootNodes.forEach((node) => {
            Object.entries(props).forEach(([key, propValue]) => {
                this.updateAttribute(node, key, propValue);
            });
            if (node.childNodes && node.childNodes.length === 0 && linkText) {
                node.textContent = linkText;
            }
        });
    }
    updateAttribute(node, key, propValue) {
        if (typeof propValue !== 'string' || !propValue || propValue === '') {
            return;
        }
        if (key === 'href') {
            const isInvalidLink = !propValue || /^https?:\/\/$/.test(propValue);
            if (isInvalidLink) {
                if (!node.href) {
                    return;
                }
                propValue = node.href;
            }
            this.renderer.setAttribute(node, key, propValue);
        }
        else if (key === 'class' && node.className !== '') {
            this.renderer.setAttribute(node, key, `${node.className} ${propValue}`);
        }
        else {
            this.renderer.setAttribute(node, key, propValue);
        }
    }
    /**
     * Determines if directive should render the field as is
     * Returns true if we are in edit mode 'chromes' (field.editable is present) or field is not empty
     * or link field text is present and we are not in edit mode 'metadata'
     * The right side of the expression was added to preserve existing functionality
     */
    shouldRender() {
        return (super.shouldRender() ||
            !!((this.field?.text || this.field?.value?.text) && !this.field?.metadata));
    }
    updateView() {
        const field = this.field;
        if (this.editable && field && field.editableFirstPart && field.editableLastPart) {
            this.renderInlineWrapper(field.editableFirstPart, field.editableLastPart);
        }
        else {
            if (!this.shouldRender()) {
                super.renderEmpty();
                return;
            }
            const props = field.href ? field : field.value;
            const linkText = field.text || field.value?.text || field.href || field.value?.href;
            const anchor = props?.anchor ? `#${props.anchor}` : '';
            const href = `${props?.href}${anchor}`;
            const mergedAttrs = { ...props, ...this.attrs, href };
            delete mergedAttrs.anchor;
            this.renderMetadata(MetadataKind.Open);
            this.renderTemplate(mergedAttrs, linkText);
            this.renderMetadata(MetadataKind.Close);
        }
    }
    renderInlineWrapper(editableFirstPart, editableLastPart) {
        const span = this.renderer.createElement('span');
        span.className = 'sc-link-wrapper';
        span.innerHTML = editableFirstPart + editableLastPart;
        // assign attributes from template to inline wrapper
        const attrs = {
            ...this.getElementAttrs(),
            ...this.attrs,
        };
        Object.entries(attrs).forEach(([key, attrValue]) => this.updateAttribute(span, key, attrValue));
        this.viewContainer.createEmbeddedView(this.templateRef);
        const parentNode = this.renderer.parentNode(this.elementRef.nativeElement);
        this.renderer.insertBefore(parentNode, span, this.elementRef.nativeElement);
        this.inlineRef = span;
    }
    getElementAttrs() {
        const view = this.templateRef.createEmbeddedView(null);
        const element = view.rootNodes[0];
        if (!element) {
            view.destroy();
            return {};
        }
        const attrs = {};
        for (let i = 0; i < element.attributes.length; i++) {
            const attr = element.attributes.item(i);
            if (attr) {
                attrs[attr.name] = attr.value;
            }
        }
        view.destroy();
        return attrs;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: LinkDirective, deps: [{ token: i0.ViewContainerRef }, { token: i0.TemplateRef }, { token: i0.Renderer2 }, { token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.13", type: LinkDirective, selector: "[scLink]", inputs: { editable: ["scLinkEditable", "editable"], attrs: ["scLinkAttrs", "attrs"], field: ["scLink", "field"], emptyFieldEditingTemplate: ["scLinkEmptyFieldEditingTemplate", "emptyFieldEditingTemplate"] }, usesInheritance: true, usesOnChanges: true, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: LinkDirective, decorators: [{
            type: Directive,
            args: [{ selector: '[scLink]' }]
        }], ctorParameters: () => [{ type: i0.ViewContainerRef }, { type: i0.TemplateRef }, { type: i0.Renderer2 }, { type: i0.ElementRef }], propDecorators: { editable: [{
                type: Input,
                args: ['scLinkEditable']
            }], attrs: [{
                type: Input,
                args: ['scLinkAttrs']
            }], field: [{
                type: Input,
                args: ['scLink']
            }], emptyFieldEditingTemplate: [{
                type: Input,
                args: ['scLinkEmptyFieldEditingTemplate']
            }] } });

class RouterLinkDirective extends LinkDirective {
    router;
    editable = true;
    attrs = {};
    field;
    /**
     * Custom template to render in Pages in Metadata edit mode if field value is empty
     */
    emptyFieldEditingTemplate;
    constructor(viewContainer, templateRef, renderer, elementRef, router) {
        super(viewContainer, templateRef, renderer, elementRef);
        this.router = router;
    }
    renderTemplate(props, linkText) {
        const viewRef = this.viewContainer.createEmbeddedView(this.templateRef);
        viewRef.rootNodes.forEach((node) => {
            Object.entries(props).forEach(([key, propValue]) => {
                this.updateAttribute(node, key, propValue);
                if (key === 'href') {
                    this.renderer.listen(node, 'click', (event) => {
                        this.router.navigateByUrl(propValue);
                        // shouldn't prevent default if the link includes a fragment
                        if (!propValue.includes('#')) {
                            event.preventDefault();
                        }
                    });
                }
            });
            if (node.childNodes && node.childNodes.length === 0 && linkText) {
                node.textContent = linkText;
            }
        });
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: RouterLinkDirective, deps: [{ token: i0.ViewContainerRef }, { token: i0.TemplateRef }, { token: i0.Renderer2 }, { token: i0.ElementRef }, { token: i1.Router }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.13", type: RouterLinkDirective, selector: "[scRouterLink]", inputs: { editable: ["scRouterLinkEditable", "editable"], attrs: ["scRouterLinkAttrs", "attrs"], field: ["scRouterLink", "field"], emptyFieldEditingTemplate: ["scRouterLinkEmptyFieldEditingTemplate", "emptyFieldEditingTemplate"] }, usesInheritance: true, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: RouterLinkDirective, decorators: [{
            type: Directive,
            args: [{ selector: '[scRouterLink]' }]
        }], ctorParameters: () => [{ type: i0.ViewContainerRef }, { type: i0.TemplateRef }, { type: i0.Renderer2 }, { type: i0.ElementRef }, { type: i1.Router }], propDecorators: { editable: [{
                type: Input,
                args: ['scRouterLinkEditable']
            }], attrs: [{
                type: Input,
                args: ['scRouterLinkAttrs']
            }], field: [{
                type: Input,
                args: ['scRouterLink']
            }], emptyFieldEditingTemplate: [{
                type: Input,
                args: ['scRouterLinkEmptyFieldEditingTemplate']
            }] } });

class GenericLinkDirective extends LinkDirective {
    router;
    editable = true;
    attrs = {};
    field;
    extras;
    /**
     * Custom template to render in Pages in Metadata edit mode if field value is empty
     */
    emptyFieldEditingTemplate;
    constructor(viewContainer, templateRef, renderer, elementRef, router) {
        super(viewContainer, templateRef, renderer, elementRef);
        this.router = router;
    }
    renderTemplate(props, linkText) {
        const viewRef = this.viewContainer.createEmbeddedView(this.templateRef);
        viewRef.rootNodes.forEach((node) => {
            Object.entries(props).forEach(([key, propValue]) => {
                if (key === 'href' && !isAbsoluteUrl(propValue)) {
                    const fragments = propValue.split('#');
                    const url = fragments[0];
                    const anchor = fragments[1];
                    const urlTree = this.router.createUrlTree([url], {
                        fragment: anchor,
                        ...this.extras,
                    });
                    this.updateAttribute(node, key, this.router.serializeUrl(urlTree));
                    this.renderer.listen(node, 'click', (event) => {
                        this.router.navigate([url], {
                            fragment: anchor,
                            ...this.extras,
                        });
                        // shouldn't prevent default if the link includes a fragment
                        if (!anchor) {
                            event.preventDefault();
                        }
                    });
                }
                else {
                    this.updateAttribute(node, key, propValue);
                }
            });
            if (node.childNodes && node.childNodes.length === 0 && linkText) {
                node.textContent = linkText;
            }
        });
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: GenericLinkDirective, deps: [{ token: i0.ViewContainerRef }, { token: i0.TemplateRef }, { token: i0.Renderer2 }, { token: i0.ElementRef }, { token: i1.Router }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.13", type: GenericLinkDirective, selector: "[scGenericLink]", inputs: { editable: ["scGenericLinkEditable", "editable"], attrs: ["scGenericLinkAttrs", "attrs"], field: ["scGenericLink", "field"], extras: ["scGenericLinkExtras", "extras"], emptyFieldEditingTemplate: ["scGenericLinkEmptyFieldEditingTemplate", "emptyFieldEditingTemplate"] }, usesInheritance: true, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: GenericLinkDirective, decorators: [{
            type: Directive,
            args: [{ selector: '[scGenericLink]' }]
        }], ctorParameters: () => [{ type: i0.ViewContainerRef }, { type: i0.TemplateRef }, { type: i0.Renderer2 }, { type: i0.ElementRef }, { type: i1.Router }], propDecorators: { editable: [{
                type: Input,
                args: ['scGenericLinkEditable']
            }], attrs: [{
                type: Input,
                args: ['scGenericLinkAttrs']
            }], field: [{
                type: Input,
                args: ['scGenericLink']
            }], extras: [{
                type: Input,
                args: ['scGenericLinkExtras']
            }], emptyFieldEditingTemplate: [{
                type: Input,
                args: ['scGenericLinkEmptyFieldEditingTemplate']
            }] } });

/**
 * Error thrown when a JssCanActivate guard returns a redirect value.
 * The redirect value will be used to redirect the user to a different route.
 */
class JssCanActivateRedirectError extends Error {
    message;
    redirectValue;
    constructor(message, redirectValue) {
        super(message);
        this.message = message;
        this.redirectValue = redirectValue;
        this.name = 'JssCanActivateRedirectError';
    }
}

/** Registers a statically loaded component */
class ComponentNameAndType {
    name;
    type;
    canActivate;
    resolve;
}
/**
 * @param {unknown} object
 */
function instanceOfComponentNameAndType(object) {
    return typeof object === 'object' && object !== null && 'type' in object;
}
/**
 * @param {unknown} object
 */
function instanceOfComponentNameAndModule(object) {
    return typeof object === 'object' && object !== null && 'module' in object;
}
const PLACEHOLDER_COMPONENTS = new InjectionToken('Sc.placeholder.components');
const PLACEHOLDER_LAZY_COMPONENTS = new InjectionToken('Sc.placeholder.lazyComponents');
const PLACEHOLDER_MISSING_COMPONENT_COMPONENT = new InjectionToken('Sc.placeholder.missingComponentComponent');
const PLACEHOLDER_HIDDEN_RENDERING_COMPONENT = new InjectionToken('Sc.placeholder.hiddenRenderingComponent');
const DYNAMIC_COMPONENT = new InjectionToken('Sc.placeholder.dynamicComponent');
const GUARD_RESOLVER = new InjectionToken('Sc.placeholder.guardResolver');
const DATA_RESOLVER = new InjectionToken('Sc.placeholder.dataResolver');

class PlaceholderLoadingDirective {
    templateRef;
    constructor(templateRef) {
        this.templateRef = templateRef;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: PlaceholderLoadingDirective, deps: [{ token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.13", type: PlaceholderLoadingDirective, selector: "[scPlaceholderLoading]", ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: PlaceholderLoadingDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[scPlaceholderLoading]',
                }]
        }], ctorParameters: () => [{ type: i0.TemplateRef }] });

class RenderEachDirective {
    templateRef;
    constructor(templateRef) {
        this.templateRef = templateRef;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: RenderEachDirective, deps: [{ token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.13", type: RenderEachDirective, selector: "[renderEach]", ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: RenderEachDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[renderEach]',
                }]
        }], ctorParameters: () => [{ type: i0.TemplateRef }] });

class RenderEmptyDirective {
    templateRef;
    constructor(templateRef) {
        this.templateRef = templateRef;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: RenderEmptyDirective, deps: [{ token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.13", type: RenderEmptyDirective, selector: "[renderEmpty]", ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: RenderEmptyDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[renderEmpty]',
                }]
        }], ctorParameters: () => [{ type: i0.TemplateRef }] });

/**
 * @param {HtmlElementRendering | ComponentRendering} rendering
 */
function isRawRendering(rendering) {
    return (!rendering.componentName &&
        rendering.name !== undefined);
}

class RawComponent {
    renderer;
    elementRef;
    rendering;
    constructor(renderer, elementRef) {
        this.renderer = renderer;
        this.elementRef = elementRef;
    }
    ngOnInit() {
        const el = this.renderer.createElement(this.rendering.name);
        const contents = this.renderer.createText(this.rendering.contents || '');
        const attributes = this.rendering.attributes;
        for (const attr in attributes) {
            // eslint-disable-next-line no-prototype-builtins
            if (attributes.hasOwnProperty(attr)) {
                const value = attributes[attr];
                this.renderer.setAttribute(el, attr, value || '');
            }
        }
        this.renderer.appendChild(el, contents);
        const parentNode = this.renderer.parentNode(this.elementRef.nativeElement);
        this.renderer.insertBefore(parentNode, el, this.elementRef.nativeElement);
        parentNode.removeChild(this.elementRef.nativeElement);
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: RawComponent, deps: [{ token: i0.Renderer2 }, { token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: RawComponent, selector: "sc-raw", inputs: { rendering: "rendering" }, ngImport: i0, template: '', isInline: true });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: RawComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'sc-raw',
                    template: '',
                }]
        }], ctorParameters: () => [{ type: i0.Renderer2 }, { type: i0.ElementRef }], propDecorators: { rendering: [{
                type: Input
            }] } });

class JssComponentFactoryService {
    injector;
    components;
    lazyComponents;
    componentMap;
    lazyComponentMap;
    constructor(injector, components, lazyComponents) {
        this.injector = injector;
        this.components = components;
        this.lazyComponents = lazyComponents;
        this.componentMap = new Map();
        this.lazyComponentMap = new Map();
        this.components.forEach((c) => this.componentMap.set(c.name, c));
        if (this.lazyComponents) {
            this.lazyComponents.forEach((c) => this.lazyComponentMap.set(c.path, c));
        }
    }
    getComponent(component) {
        const loadedComponent = this.componentMap.get(component.componentName);
        if (loadedComponent) {
            return Promise.resolve({
                componentDefinition: this.applySXAParams(component),
                componentImplementation: loadedComponent.type,
                canActivate: loadedComponent.canActivate,
                resolve: loadedComponent.resolve,
            });
        }
        const lazyComponent = this.lazyComponentMap.get(component.componentName);
        if (lazyComponent) {
            return lazyComponent.loadChildren().then((lazyChild) => {
                let componentType = null;
                const moduleRef = createNgModule(lazyChild, this.injector);
                const dynamicComponentType = moduleRef.injector.get(DYNAMIC_COMPONENT);
                if (!dynamicComponentType) {
                    throw new Error(`JssComponentFactoryService: Lazy load module for component "${lazyComponent.path}" missing DYNAMIC_COMPONENT provider. Missing JssModule.forChild()?`);
                }
                if (component.componentName in dynamicComponentType) {
                    // eslint-disable-next-line @typescript-eslint/no-explicit-any
                    componentType = dynamicComponentType[component.componentName];
                }
                else {
                    if (typeof dynamicComponentType === 'function') {
                        componentType = dynamicComponentType;
                    }
                    else {
                        throw new Error(`JssComponentFactoryService: Lazy load module for component "${lazyComponent.path}" missing DYNAMIC_COMPONENT provider. Missing JssModule.forChild()?`);
                    }
                }
                return {
                    componentDefinition: this.applySXAParams(component),
                    componentImplementation: componentType,
                    componentModuleRef: moduleRef,
                    canActivate: lazyComponent.canActivate,
                    resolve: lazyComponent.resolve,
                };
            });
        }
        return Promise.resolve({
            componentDefinition: component,
        });
    }
    getComponents(components) {
        // acquire all components and keep them in order while handling their potential async-ness
        return Promise.all(components.map((component) => isRawRendering(component) ? this.getRawComponent(component) : this.getComponent(component)));
    }
    getRawComponent(component) {
        return Promise.resolve({
            componentImplementation: RawComponent,
            componentDefinition: component,
        });
    }
    applySXAParams(rendering) {
        // Provide aggregated SXA styles on params 'styles'
        const styles = [];
        if (rendering.params?.GridParameters) {
            styles.push(rendering.params.GridParameters.trim());
        }
        if (rendering.params?.Styles) {
            styles.push(rendering.params.Styles.trim());
        }
        if (rendering.params && styles.length > 0) {
            rendering.params.styles = styles.join(' ');
        }
        return rendering;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: JssComponentFactoryService, deps: [{ token: i0.Injector }, { token: PLACEHOLDER_COMPONENTS }, { token: PLACEHOLDER_LAZY_COMPONENTS }], target: i0.ɵɵFactoryTarget.Injectable });
    static ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: JssComponentFactoryService });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: JssComponentFactoryService, decorators: [{
            type: Injectable
        }], ctorParameters: () => [{ type: i0.Injector }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [PLACEHOLDER_COMPONENTS]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [PLACEHOLDER_LAZY_COMPONENTS]
                }] }] });

class BaseJssState {
    sitecore;
    viewBag;
}
/**
 * The service that is used to store and retrieve the current state of the app.
 * Referenced by the SDK to get the current state of the app.
 */
class JssStateService {
    transferState;
    _state;
    constructor(transferState) {
        this.transferState = transferState;
        this._state = new BehaviorSubject({});
    }
    /**
     * Observable JSS state to subscribe to
     */
    get state() {
        return this._state.asObservable();
    }
    /**
     * The 'here and now' state value, usable without subscribing
     */
    get stateValue() {
        return this._state.value;
    }
    /**
     * Set new JSS state
     * @param {JssState} newState new state
     */
    setState(newState) {
        this._state.next(newState);
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: JssStateService, deps: [{ token: i0.TransferState }], target: i0.ɵɵFactoryTarget.Injectable });
    static ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: JssStateService });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: JssStateService, decorators: [{
            type: Injectable
        }], ctorParameters: () => [{ type: i0.TransferState }] });

/* eslint-disable @angular-eslint/no-conflicting-lifecycle */
class PlaceholderComponent {
    differs;
    componentFactory;
    changeDetectorRef;
    elementRef;
    renderer;
    router;
    missingComponentComponent;
    hiddenRenderingComponent;
    guardResolver;
    dataResolver;
    platformId;
    jssState;
    name;
    rendering;
    renderings;
    outputs;
    clientOnly = false;
    loaded = new EventEmitter();
    failed = new EventEmitter();
    renderEachTemplate;
    renderEmptyTemplate;
    placeholderLoading;
    view;
    metadataNode;
    isLoading = true;
    metadataMode;
    chromeType;
    _inputs;
    _differ;
    _componentInstances = [];
    placeholderData;
    destroyed = false;
    parentStyleAttribute = '';
    contextSubscription;
    constructor(differs, componentFactory, changeDetectorRef, elementRef, renderer, router, missingComponentComponent, hiddenRenderingComponent, guardResolver, dataResolver, platformId, jssState) {
        this.differs = differs;
        this.componentFactory = componentFactory;
        this.changeDetectorRef = changeDetectorRef;
        this.elementRef = elementRef;
        this.renderer = renderer;
        this.router = router;
        this.missingComponentComponent = missingComponentComponent;
        this.hiddenRenderingComponent = hiddenRenderingComponent;
        this.guardResolver = guardResolver;
        this.dataResolver = dataResolver;
        this.platformId = platformId;
        this.jssState = jssState;
        this.contextSubscription = this.jssState.state.subscribe(({ sitecore }) => {
            this.metadataMode = sitecore?.context.editMode === EditMode.Metadata;
        });
    }
    set inputs(value) {
        this._inputs = value;
        if (!this._differ && value) {
            this._differ = this.differs.find(value).create();
        }
    }
    ngOnInit() {
        this.chromeType = this.name ? 'placeholder' : 'rendering';
        // just to ensure the element exists
        const elem = this.elementRef.nativeElement;
        if (elem) {
            const attributes = elem.attributes;
            for (let i = 0; i < attributes.length; i++) {
                const attr = attributes.item(i);
                if (attr && attr.name.indexOf('_ngcontent') !== -1) {
                    this.parentStyleAttribute = attr.name;
                }
            }
        }
        this.placeholderData = this.renderings || this.getPlaceholder() || [];
    }
    ngOnDestroy() {
        this.destroyed = true;
        this._componentInstances = [];
        if (this.contextSubscription) {
            this.contextSubscription.unsubscribe();
        }
    }
    ngOnChanges(changes) {
        this.chromeType = changes.name ? 'placeholder' : 'rendering';
        if (changes.rendering || changes.renderings) {
            this.placeholderData = this.renderings || this.getPlaceholder() || [];
            this._render();
        }
    }
    ngDoCheck() {
        if (!this._differ || !this._inputs || this._componentInstances.length === 0) {
            return;
        }
        const changes = this._differ.diff(this._inputs);
        if (!changes) {
            return;
        }
        const updates = {};
        changes.forEachRemovedItem((change) => (updates[change.key] = null));
        changes.forEachAddedItem((change) => (updates[change.key] = change.currentValue));
        changes.forEachChangedItem((change) => (updates[change.key] = change.currentValue));
        this._componentInstances.forEach((componentInstance) => this._setComponentInputs(componentInstance, updates));
    }
    /**
     * Gets id for Metadata code blocks, in specific format
     * Metadata code blocks will wrap be added around placeholder content and each rendering component
     * to allow for editing integration in Pages.
     * @param {string} kind code block type ("open" or "close"). "open" is added before an element, and "close" added after one.
     * @param {string?} renderingId rendering uid to apply as id to code block
     * @returns {string} formatted id value for code HTML node
     */
    getCodeBlockId = (kind, renderingId) => {
        if (this.rendering && kind === MetadataKind.Open) {
            const placeholderName = this.name;
            const id = renderingId || this.rendering?.uid;
            if (!renderingId && placeholderName) {
                let phId = '';
                for (const placeholder of Object.keys(this.rendering.placeholders || [])) {
                    if (placeholderName === placeholder) {
                        phId = id
                            ? `${placeholderName}_${id}`
                            : `${placeholderName}_${DEFAULT_PLACEHOLDER_UID}`;
                        break;
                    }
                    // Check if the placeholder is a dynamic placeholder
                    if (isDynamicPlaceholder(placeholder)) {
                        const pattern = getDynamicPlaceholderPattern(placeholder);
                        // Check if the placeholder matches the dynamic placeholder pattern
                        if (pattern.test(placeholderName)) {
                            phId = id ? `${placeholder}_${id}` : `${placeholder}_${DEFAULT_PLACEHOLDER_UID}`;
                            break;
                        }
                    }
                }
                return phId;
            }
            else {
                return id;
            }
        }
        return undefined;
    };
    /**
     * Get renderings/components to be rendered for current placeholder name
     * Can modify the inner placeholders collection to adjust to using SXA dynamic placeholders
     * @returns {ComponentRendering<ComponentFields> | HtmlElementRendering[] | null} List of renderings to be rendered
     */
    getPlaceholder() {
        let phName = this.name?.slice() || '';
        /**
         * Process (SXA) dynamic placeholders
         * Find and replace the matching dynamic placeholder e.g 'nameOfContainer-{*}' with the requested e.g. 'nameOfContainer-1'.
         * For Metadata EditMode, we need to keep the raw placeholder name in place.
         */
        this.rendering?.placeholders &&
            Object.keys(this.rendering.placeholders).forEach((placeholder) => {
                const patternPlaceholder = isDynamicPlaceholder(placeholder)
                    ? getDynamicPlaceholderPattern(placeholder)
                    : null;
                if (patternPlaceholder && patternPlaceholder.test(phName)) {
                    if (this.metadataMode) {
                        phName = placeholder;
                    }
                    else {
                        this.rendering.placeholders[phName] = this.rendering.placeholders[placeholder];
                        delete this.rendering.placeholders[placeholder];
                    }
                }
            });
        if (this.rendering &&
            this.rendering.placeholders &&
            Object.keys(this.rendering.placeholders).length > 0) {
            return this.rendering.placeholders[phName];
        }
        return null;
    }
    _setComponentInputs(componentInstance, inputs) {
        Object.entries(inputs).forEach(([input, inputValue]) => (componentInstance[input] = inputValue));
    }
    _subscribeComponentOutputs(componentInstance, outputs) {
        Object.keys(outputs)
            .filter((output) => componentInstance[output] && componentInstance[output] instanceof Observable)
            .forEach((output) => componentInstance[output]
            .pipe(takeWhile(() => !this.destroyed))
            .subscribe(outputs[output]));
    }
    async _render() {
        if (this.clientOnly && isPlatformServer(this.platformId)) {
            return;
        }
        this._componentInstances = [];
        this.view.clear();
        if (!this.rendering && !this.renderings) {
            return;
        }
        if (!this.name && !this.renderings) {
            console.warn('Placeholder name was not specified, and explicit renderings array was not passed. Placeholder requires either name and rendering, or renderings.');
            this.isLoading = false;
            return;
        }
        const placeholder = this.placeholderData;
        if (!placeholder) {
            console.warn(`Placeholder '${this.name}' was not found in the current rendering data`, JSON.stringify(this.rendering, null, 2));
            this.isLoading = false;
            return;
        }
        // if the placeholder is empty (contains only raw renderings), then we may need to use the empty template if it's defined
        const placeholderIsEmpty = placeholder.every((rendering) => isRawRendering(rendering));
        if (this.renderEmptyTemplate && placeholderIsEmpty) {
            this.view.createEmbeddedView(this.renderEmptyTemplate.templateRef, {
                renderings: placeholder,
            });
            this.isLoading = false;
        }
        else {
            const factories = await this.componentFactory.getComponents(placeholder);
            try {
                const nonGuarded = await this.guardResolver(factories);
                const withData = await this.dataResolver(nonGuarded);
                // not using index to ensure code blocks are rendered at correct positions
                withData.forEach((rendering) => {
                    this.metadataMode &&
                        this.view.createEmbeddedView(this.metadataNode, {
                            kind: MetadataKind.Open,
                            chromeType: 'rendering',
                            renderingId: rendering.factory.componentDefinition?.uid,
                        });
                    if (this.renderEachTemplate && !isRawRendering(rendering.factory.componentDefinition)) {
                        this._renderTemplatedComponent(rendering.factory.componentDefinition);
                    }
                    else {
                        this._renderEmbeddedComponent(rendering.factory, rendering.data);
                    }
                    this.metadataMode &&
                        this.view.createEmbeddedView(this.metadataNode, {
                            kind: MetadataKind.Close,
                            chromeType: 'rendering',
                            renderingId: rendering.factory.componentDefinition?.uid,
                        });
                });
                this.isLoading = false;
                this.changeDetectorRef.markForCheck();
                this.loaded.emit(this.name);
            }
            catch (e) {
                this.isLoading = false;
                if (e instanceof JssCanActivateRedirectError) {
                    const redirectValue = e.redirectValue;
                    if (redirectValue instanceof RedirectCommand) {
                        this.router.navigateByUrl(redirectValue.redirectTo);
                    }
                    else if (redirectValue instanceof UrlTree) {
                        this.router.navigateByUrl(redirectValue);
                    }
                    else if (typeof redirectValue === 'string') {
                        this.router.navigate([redirectValue]);
                    }
                    else {
                        this.router.navigate(redirectValue);
                    }
                }
                else {
                    this.failed.emit(e);
                    console.warn(`Placeholder '${this.name}' was not able to render with the current rendering data and error`, JSON.stringify(this.rendering, null, 2), e);
                    return;
                }
            }
        }
    }
    _renderTemplatedComponent(rendering) {
        // the render-each template takes care of all component mapping etc
        // generally using <sc-render-component> which is about like _renderEmbeddedComponent()
        // as a separate component
        this.view.createEmbeddedView(this.renderEachTemplate.templateRef, {
            rendering,
        });
    }
    _renderEmbeddedComponent(rendering, data) {
        if (rendering.componentDefinition.componentName ===
            constants.HIDDEN_RENDERING_NAME) {
            rendering.componentImplementation = this.hiddenRenderingComponent;
        }
        if (!rendering.componentImplementation) {
            const componentName = rendering.componentDefinition.componentName;
            console.error(`Placeholder ${this.name} contains unknown component ${componentName}.`, `Ensure component is mapped, like:
        JssModule.withComponents([
          { name: '${componentName}', type: ${componentName}Component }
        ])`);
            rendering.componentImplementation = this.missingComponentComponent;
        }
        // apply the parent style attribute _ngcontent
        // work-around for https://github.com/angular/angular/issues/12215
        const createdComponentRef = this.view.createComponent(rendering.componentImplementation, {
            ngModuleRef: rendering.componentModuleRef,
        });
        if (this.parentStyleAttribute) {
            this.renderer.setAttribute(createdComponentRef.location.nativeElement, this.parentStyleAttribute, '');
        }
        const componentInstance = createdComponentRef.instance;
        componentInstance.rendering = rendering.componentDefinition;
        componentInstance.data = data;
        if (this._inputs) {
            this._setComponentInputs(componentInstance, this._inputs);
        }
        if (this.outputs) {
            this._subscribeComponentOutputs(componentInstance, this.outputs);
        }
        this._componentInstances.push(componentInstance);
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: PlaceholderComponent, deps: [{ token: i0.KeyValueDiffers }, { token: JssComponentFactoryService }, { token: i0.ChangeDetectorRef }, { token: i0.ElementRef }, { token: i0.Renderer2 }, { token: i1.Router }, { token: PLACEHOLDER_MISSING_COMPONENT_COMPONENT }, { token: PLACEHOLDER_HIDDEN_RENDERING_COMPONENT }, { token: GUARD_RESOLVER }, { token: DATA_RESOLVER }, { token: PLATFORM_ID }, { token: JssStateService }], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: PlaceholderComponent, selector: "sc-placeholder,[sc-placeholder]", inputs: { name: "name", rendering: "rendering", renderings: "renderings", outputs: "outputs", clientOnly: "clientOnly", inputs: "inputs" }, outputs: { loaded: "loaded", failed: "failed" }, queries: [{ propertyName: "renderEachTemplate", first: true, predicate: RenderEachDirective, descendants: true, static: true }, { propertyName: "renderEmptyTemplate", first: true, predicate: RenderEmptyDirective, descendants: true, static: true }, { propertyName: "placeholderLoading", first: true, predicate: PlaceholderLoadingDirective, descendants: true, static: true }], viewQueries: [{ propertyName: "view", first: true, predicate: ["view"], descendants: true, read: ViewContainerRef, static: true }, { propertyName: "metadataNode", first: true, predicate: ["metadataCodeBlock"], descendants: true, read: TemplateRef }], usesOnChanges: true, ngImport: i0, template: `
    <ng-template
      *ngIf="isLoading"
      [ngTemplateOutlet]="placeholderLoading?.templateRef"
    ></ng-template>
    <ng-template
      #metadataCodeBlock
      let-kind="kind"
      let-type="chromeType"
      let-renderingId="renderingId"
    >
      <code
        [attr.kind]="kind"
        type="text/sitecore"
        [attr.chrometype]="type"
        class="scpm"
        [attr.id]="getCodeBlockId(kind, renderingId)"
      ></code
    ></ng-template>

    <ng-container
      *ngTemplateOutlet="
        metadataMode && metadataCodeBlock;
        context: { kind: 'open', chromeType: 'placeholder' }
      "
    >
    </ng-container>

    <ng-template #view></ng-template>

    <ng-container
      *ngTemplateOutlet="
        metadataMode && metadataCodeBlock;
        context: { kind: 'close', chromeType: 'placeholder' }
      "
    >
    </ng-container>
  `, isInline: true, dependencies: [{ kind: "directive", type: i1$1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i1$1.NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: PlaceholderComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'sc-placeholder,[sc-placeholder]',
                    template: `
    <ng-template
      *ngIf="isLoading"
      [ngTemplateOutlet]="placeholderLoading?.templateRef"
    ></ng-template>
    <ng-template
      #metadataCodeBlock
      let-kind="kind"
      let-type="chromeType"
      let-renderingId="renderingId"
    >
      <code
        [attr.kind]="kind"
        type="text/sitecore"
        [attr.chrometype]="type"
        class="scpm"
        [attr.id]="getCodeBlockId(kind, renderingId)"
      ></code
    ></ng-template>

    <ng-container
      *ngTemplateOutlet="
        metadataMode && metadataCodeBlock;
        context: { kind: 'open', chromeType: 'placeholder' }
      "
    >
    </ng-container>

    <ng-template #view></ng-template>

    <ng-container
      *ngTemplateOutlet="
        metadataMode && metadataCodeBlock;
        context: { kind: 'close', chromeType: 'placeholder' }
      "
    >
    </ng-container>
  `,
                }]
        }], ctorParameters: () => [{ type: i0.KeyValueDiffers }, { type: JssComponentFactoryService }, { type: i0.ChangeDetectorRef }, { type: i0.ElementRef }, { type: i0.Renderer2 }, { type: i1.Router }, { type: i0.Type, decorators: [{
                    type: Inject,
                    args: [PLACEHOLDER_MISSING_COMPONENT_COMPONENT]
                }] }, { type: i0.Type, decorators: [{
                    type: Inject,
                    args: [PLACEHOLDER_HIDDEN_RENDERING_COMPONENT]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [GUARD_RESOLVER]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [DATA_RESOLVER]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [PLATFORM_ID]
                }] }, { type: JssStateService }], propDecorators: { name: [{
                type: Input
            }], rendering: [{
                type: Input
            }], renderings: [{
                type: Input
            }], outputs: [{
                type: Input
            }], clientOnly: [{
                type: Input
            }], loaded: [{
                type: Output
            }], failed: [{
                type: Output
            }], renderEachTemplate: [{
                type: ContentChild,
                args: [RenderEachDirective, { static: true }]
            }], renderEmptyTemplate: [{
                type: ContentChild,
                args: [RenderEmptyDirective, { static: true }]
            }], placeholderLoading: [{
                type: ContentChild,
                args: [PlaceholderLoadingDirective, { static: true }]
            }], view: [{
                type: ViewChild,
                args: ['view', { read: ViewContainerRef, static: true }]
            }], metadataNode: [{
                type: ViewChild,
                args: ['metadataCodeBlock', { read: TemplateRef }]
            }], inputs: [{
                type: Input
            }] } });

class HiddenRenderingComponent {
    get style() {
        return 'background-image: linear-gradient(45deg, #ffffff 25%, #dcdcdc 25%, #dcdcdc 50%, #ffffff 50%, #ffffff 75%, #dcdcdc 75%, #dcdcdc 100%); background-size: 3px 3px; display: flex; justify-content: center; align-items: center; padding: 30px; color: #aaa;';
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: HiddenRenderingComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: HiddenRenderingComponent, selector: "sc-hidden-rendering", host: { properties: { "style": "this.style" } }, ngImport: i0, template: 'The component is hidden', isInline: true });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: HiddenRenderingComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'sc-hidden-rendering',
                    template: 'The component is hidden',
                }]
        }], propDecorators: { style: [{
                type: HostBinding,
                args: ['style']
            }] } });

/**
 * Renders a single JSS component given a rendering definition.
 * Useful inside templated placeholders.
 */
class RenderComponentComponent {
    differs;
    componentFactory;
    missingComponentComponent;
    rendering;
    outputs;
    view;
    _inputs;
    _differ;
    destroyed = false;
    constructor(differs, componentFactory, missingComponentComponent) {
        this.differs = differs;
        this.componentFactory = componentFactory;
        this.missingComponentComponent = missingComponentComponent;
    }
    set inputs(value) {
        this._inputs = value;
        if (!this._differ && value) {
            this._differ = this.differs.find(value).create();
        }
    }
    ngOnChanges(changes) {
        if (changes.rendering) {
            this._render();
        }
    }
    _setComponentInputs(componentInstance, inputs) {
        Object.entries(inputs).forEach(([input, inputValue]) => (componentInstance[input] = inputValue));
    }
    _subscribeComponentOutputs(componentInstance, outputs) {
        Object.keys(outputs)
            .filter((output) => componentInstance[output] && componentInstance[output] instanceof Observable)
            .forEach((output) => componentInstance[output]
            .pipe(takeWhile(() => !this.destroyed))
            .subscribe(outputs[output]));
    }
    _render() {
        this.view.clear();
        if (!this.rendering) {
            return;
        }
        const resolveComponent = isRawRendering(this.rendering)
            ? Promise.resolve({
                componentImplementation: RawComponent,
                componentDefinition: this.rendering,
            })
            : this.componentFactory.getComponent(this.rendering);
        resolveComponent.then((rendering) => {
            if (!rendering.componentImplementation) {
                const componentName = rendering.componentDefinition.componentName;
                console.error(`Attempted to render unknown component ${componentName}.`, `Ensure component is mapped, like:
          JssModule.withComponents([
            { name: '${componentName}', type: ${componentName}Component }
          ])`);
                rendering.componentImplementation = this.missingComponentComponent;
            }
            const componentInstance = this.view.createComponent(rendering.componentImplementation)
                .instance;
            componentInstance.rendering = rendering.componentDefinition;
            if (this._inputs) {
                this._setComponentInputs(componentInstance, this._inputs);
            }
            if (this.outputs) {
                this._subscribeComponentOutputs(componentInstance, this.outputs);
            }
        });
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: RenderComponentComponent, deps: [{ token: i0.KeyValueDiffers }, { token: JssComponentFactoryService }, { token: PLACEHOLDER_MISSING_COMPONENT_COMPONENT }], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: RenderComponentComponent, selector: "sc-render-component", inputs: { rendering: "rendering", outputs: "outputs", inputs: "inputs" }, viewQueries: [{ propertyName: "view", first: true, predicate: ["view"], descendants: true, read: ViewContainerRef, static: true }], usesOnChanges: true, ngImport: i0, template: `
    <ng-template #view></ng-template>
  `, isInline: true });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: RenderComponentComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'sc-render-component',
                    template: `
    <ng-template #view></ng-template>
  `,
                }]
        }], ctorParameters: () => [{ type: i0.KeyValueDiffers }, { type: JssComponentFactoryService }, { type: i0.Type, decorators: [{
                    type: Inject,
                    args: [PLACEHOLDER_MISSING_COMPONENT_COMPONENT]
                }] }], propDecorators: { rendering: [{
                type: Input
            }], outputs: [{
                type: Input
            }], view: [{
                type: ViewChild,
                args: ['view', { read: ViewContainerRef, static: true }]
            }], inputs: [{
                type: Input
            }] } });

class DateDirective extends BaseFieldDirective {
    templateRef;
    datePipe;
    format;
    timezone;
    locale;
    editable = true;
    field;
    /**
     * Custom template to render in Pages in Metadata edit mode if field value is empty
     */
    emptyFieldEditingTemplate;
    /**
     * Default component to render in Pages in Metadata edit mode if field value is empty and emptyFieldEditingTemplate is not provided
     */
    defaultFieldEditingComponent;
    constructor(viewContainer, templateRef, datePipe) {
        super(viewContainer);
        this.templateRef = templateRef;
        this.datePipe = datePipe;
        this.defaultFieldEditingComponent = DefaultEmptyFieldEditingComponent;
    }
    ngOnChanges(changes) {
        if (changes.field || changes.format) {
            this.viewContainer.clear();
            this.updateView();
        }
    }
    updateView() {
        if (!this.shouldRender()) {
            super.renderEmpty();
            return;
        }
        this.renderMetadata(MetadataKind.Open);
        this.viewRef = this.viewContainer.createEmbeddedView(this.templateRef);
        this.renderMetadata(MetadataKind.Close);
        const field = this.field;
        const html = field.editable && this.editable ? field.editable : field.value;
        const setDangerously = field.editable && this.editable;
        this.viewRef.rootNodes.forEach((node) => {
            if (setDangerously) {
                node.innerHTML = html;
            }
            else {
                node.textContent = this.datePipe.transform(html, this.format, this.timezone, this.locale);
            }
        });
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DateDirective, deps: [{ token: i0.ViewContainerRef }, { token: i0.TemplateRef }, { token: i1$1.DatePipe }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.13", type: DateDirective, selector: "[scDate]", inputs: { format: ["scDateFormat", "format"], timezone: ["scDateTimezone", "timezone"], locale: ["scDateLocale", "locale"], editable: ["scDateEditable", "editable"], field: ["scDate", "field"], emptyFieldEditingTemplate: ["scDateEmptyFieldEditingTemplate", "emptyFieldEditingTemplate"] }, usesInheritance: true, usesOnChanges: true, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DateDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[scDate]',
                }]
        }], ctorParameters: () => [{ type: i0.ViewContainerRef }, { type: i0.TemplateRef }, { type: i1$1.DatePipe }], propDecorators: { format: [{
                type: Input,
                args: ['scDateFormat']
            }], timezone: [{
                type: Input,
                args: ['scDateTimezone']
            }], locale: [{
                type: Input,
                args: ['scDateLocale']
            }], editable: [{
                type: Input,
                args: ['scDateEditable']
            }], field: [{
                type: Input,
                args: ['scDate']
            }], emptyFieldEditingTemplate: [{
                type: Input,
                args: ['scDateEmptyFieldEditingTemplate']
            }] } });

class EditFrameComponent {
    dataSource;
    buttons;
    title;
    tooltip;
    cssClass;
    parameters;
    sitecore;
    isEditing = false;
    frameProps = {};
    chromeData = '';
    ngOnChanges() {
        this.isEditing = this.sitecore.context.pageEditing || false;
        if (!this.isEditing) {
            return;
        }
        this.frameProps.class = 'scLooseFrameZone';
        if (this.cssClass) {
            this.frameProps.class = `${this.frameProps.class} ${this.cssClass}`;
        }
        // item uri for edit frame target
        if (this.dataSource) {
            const route = this.sitecore.route;
            const databaseName = this.dataSource.databaseName || route?.databaseName;
            const language = this.dataSource.language || this.sitecore.context.language;
            this.frameProps.sc_item = `sitecore://${databaseName}/${this.dataSource.itemId}?lang=${language}`;
        }
        this.chromeData = this.buildChromeData();
    }
    buildChromeData() {
        const chromeData = {
            displayName: this.title,
            expandedDisplayName: this.tooltip,
        };
        if (this.dataSource) {
            chromeData.contextItemUri = this.frameProps.sc_item;
        }
        chromeData.commands = this.buttons?.map((value) => {
            return mapButtonToCommand(value, this.dataSource?.itemId, this.parameters);
        });
        return JSON.stringify(chromeData);
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: EditFrameComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: EditFrameComponent, selector: "sc-edit-frame,[sc-edit-frame]", inputs: { dataSource: "dataSource", buttons: "buttons", title: "title", tooltip: "tooltip", cssClass: "cssClass", parameters: "parameters", sitecore: "sitecore" }, usesOnChanges: true, ngImport: i0, template: `
    <ng-template #childContent>
      <ng-content></ng-content>
    </ng-template>
    <div
      *ngIf="isEditing; else elseBlock"
      [class]="frameProps.class"
      [attr.sc_item]="frameProps.sc_item"
    >
      <span class="scChromeData">{{ chromeData }}</span>
      <ng-container *ngTemplateOutlet="childContent"></ng-container>
    </div>
    <ng-template #elseBlock>
      <ng-container *ngTemplateOutlet="childContent"></ng-container>
    </ng-template>
  `, isInline: true, dependencies: [{ kind: "directive", type: i1$1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i1$1.NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: EditFrameComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'sc-edit-frame,[sc-edit-frame]',
                    template: `
    <ng-template #childContent>
      <ng-content></ng-content>
    </ng-template>
    <div
      *ngIf="isEditing; else elseBlock"
      [class]="frameProps.class"
      [attr.sc_item]="frameProps.sc_item"
    >
      <span class="scChromeData">{{ chromeData }}</span>
      <ng-container *ngTemplateOutlet="childContent"></ng-container>
    </div>
    <ng-template #elseBlock>
      <ng-container *ngTemplateOutlet="childContent"></ng-container>
    </ng-template>
  `,
                }]
        }], propDecorators: { dataSource: [{
                type: Input
            }], buttons: [{
                type: Input
            }], title: [{
                type: Input
            }], tooltip: [{
                type: Input
            }], cssClass: [{
                type: Input
            }], parameters: [{
                type: Input
            }], sitecore: [{
                type: Input
            }] } });

/**
 * Component that renders editing scripts and client data for the current page in Sitecore Editor.
 * Only renders scripts when Metadata mode is used.
 */
class EditingScriptsComponent {
    renderer;
    stateService;
    document;
    constructor(renderer, stateService, document) {
        this.renderer = renderer;
        this.stateService = stateService;
        this.document = document;
    }
    ngOnInit() {
        const state = this.stateService.stateValue;
        const { pageState, editMode, clientData, clientScripts } = state.sitecore?.context || {};
        // Don't render anything if not in editing mode or not server side
        if (pageState === LayoutServicePageState.Normal ||
            pageState === LayoutServicePageState.Preview ||
            !isServer()) {
            return;
        }
        if (editMode === EditMode.Metadata) {
            const jssClientData = { ...clientData, ...getJssPagesClientData() };
            clientScripts?.forEach((src) => {
                const scriptElement = this.renderer.createElement('script');
                scriptElement.src = src;
                this.renderer.appendChild(this.document.body, scriptElement);
            });
            Object.keys(jssClientData).forEach((id) => {
                const scriptElement = this.renderer.createElement('script');
                scriptElement.id = id;
                scriptElement.type = 'application/json';
                scriptElement.innerHTML = JSON.stringify(jssClientData[id]);
                this.renderer.appendChild(this.document.body, scriptElement);
            });
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: EditingScriptsComponent, deps: [{ token: i0.Renderer2 }, { token: JssStateService }, { token: DOCUMENT }], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: EditingScriptsComponent, selector: "sc-editing-scripts", ngImport: i0, template: '', isInline: true });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: EditingScriptsComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'sc-editing-scripts',
                    template: '',
                }]
        }], ctorParameters: () => [{ type: i0.Renderer2 }, { type: JssStateService }, { type: Document, decorators: [{
                    type: Inject,
                    args: [DOCUMENT]
                }] }] });

/**
 * Injection token for the Sitecore Edge configuration.
 */
const EDGE_CONFIG = new InjectionToken('Sc.edgeConfig');

/* eslint-disable prefer-const */
let { executeScriptElements, loadForm, subscribeToFormSubmitEvent } = form;
/**
 * Mock function to replace the form module functions for `testing` purposes.
 * @param {any} formModule - The form module to mock
 */
const mockFormModule = (formModule) => {
    executeScriptElements = formModule.executeScriptElements;
    loadForm = formModule.loadForm;
    subscribeToFormSubmitEvent = formModule.subscribeToFormSubmitEvent;
};
/**
 * A component that renders a Sitecore Form.
 * It fetches the form markup from the Sitecore Edge service and renders it in the component's template.
 */
class FormComponent {
    edgeConfig;
    platformId;
    elRef;
    jssState;
    /**
     * The rendering data for the component
     */
    rendering;
    hasError = false;
    isEditing = false;
    contextSubscription;
    constructor(edgeConfig, platformId, elRef, jssState) {
        this.edgeConfig = edgeConfig;
        this.platformId = platformId;
        this.elRef = elRef;
        this.jssState = jssState;
    }
    ngOnInit() {
        if (isPlatformBrowser(this.platformId)) {
            this.loadForm();
            this.contextSubscription = this.jssState.state.subscribe(({ sitecore }) => {
                this.isEditing = sitecore?.context.pageState !== LayoutServicePageState.Normal;
            });
        }
    }
    ngOnDestroy() {
        if (this.contextSubscription) {
            this.contextSubscription.unsubscribe();
        }
    }
    /**
     * Fetches the form markup from the Sitecore Edge service and renders it in the component's template.
     */
    async loadForm() {
        const { sitecoreEdgeContextId, sitecoreEdgeUrl } = this.edgeConfig;
        try {
            const content = await loadForm(sitecoreEdgeContextId, this.rendering.params.FormId, sitecoreEdgeUrl);
            this.elRef.nativeElement.innerHTML = content;
            this.elRef.nativeElement.className = this.rendering.params?.styles?.trimEnd() || '';
            this.elRef.nativeElement.id = this.rendering.params?.RenderingIdentifier || '';
            const form = this.elRef.nativeElement.querySelector('form');
            if (!form) {
                debug.form(`Form '${this.rendering.params.FormId}' was not able to render since form element was not found`);
                return;
            }
            executeScriptElements(this.elRef.nativeElement);
            if (!this.isEditing) {
                subscribeToFormSubmitEvent(form, this.rendering.uid);
            }
        }
        catch {
            this.hasError = true;
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: FormComponent, deps: [{ token: EDGE_CONFIG }, { token: PLATFORM_ID }, { token: i0.ElementRef }, { token: JssStateService }], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: FormComponent, selector: "app-form", inputs: { rendering: "rendering" }, ngImport: i0, template: `
    <ng-container *ngIf="isEditing">
      <ng-container *ngIf="!rendering.params.FormId">
        <div
          style="background: darkorange; outline: 5px solid orange; padding: 10px; color: white; max-width: 500px;"
        >
          <h2>{{ rendering.componentName }}</h2>
          <p>JSS component is missing FormId rendering parameter.</p>
        </div>
      </ng-container>
      <ng-container *ngIf="hasError">
        <div class="sc-jss-placeholder-error">There was a problem loading this section</div>
      </ng-container>
    </ng-container>
  `, isInline: true, dependencies: [{ kind: "directive", type: i1$1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: FormComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'app-form',
                    template: `
    <ng-container *ngIf="isEditing">
      <ng-container *ngIf="!rendering.params.FormId">
        <div
          style="background: darkorange; outline: 5px solid orange; padding: 10px; color: white; max-width: 500px;"
        >
          <h2>{{ rendering.componentName }}</h2>
          <p>JSS component is missing FormId rendering parameter.</p>
        </div>
      </ng-container>
      <ng-container *ngIf="hasError">
        <div class="sc-jss-placeholder-error">There was a problem loading this section</div>
      </ng-container>
    </ng-container>
  `,
                }]
        }], ctorParameters: () => [{ type: undefined, decorators: [{
                    type: Inject,
                    args: [EDGE_CONFIG]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [PLATFORM_ID]
                }] }, { type: i0.ElementRef }, { type: JssStateService }], propDecorators: { rendering: [{
                type: Input
            }] } });

class RichTextDirective extends BaseFieldDirective {
    templateRef;
    renderer;
    router;
    editable = true;
    field;
    /**
     * Custom template to render in Pages in Metadata edit mode if field value is empty
     */
    emptyFieldEditingTemplate;
    /**
     * Default component to render in Pages in Metadata edit mode if field value is empty and emptyFieldEditingTemplate is not provided
     */
    defaultFieldEditingComponent;
    constructor(viewContainer, templateRef, renderer, router) {
        super(viewContainer);
        this.templateRef = templateRef;
        this.renderer = renderer;
        this.router = router;
        this.defaultFieldEditingComponent = DefaultEmptyFieldEditingComponent;
    }
    ngOnChanges(changes) {
        if (changes.field || changes.editable) {
            this.viewContainer.clear();
            this.updateView();
        }
    }
    updateView() {
        if (!this.shouldRender()) {
            super.renderEmpty();
            return;
        }
        this.renderMetadata(MetadataKind.Open);
        this.viewRef = this.viewContainer.createEmbeddedView(this.templateRef);
        this.renderMetadata(MetadataKind.Close);
        const field = this.field;
        const html = field.editable && this.editable ? field.editable : field.value;
        this.viewRef.rootNodes.forEach((node) => {
            node.innerHTML = html;
            if (!node.querySelectorAll) {
                return;
            }
            const links = node.querySelectorAll('a[href]');
            const linksArray = [].slice.call(links);
            linksArray.forEach((link) => {
                const href = link.getAttribute('href');
                const target = link.getAttribute('target');
                if (!href || isAbsoluteUrl(href) || target === '_blank' || target === '_top') {
                    return;
                }
                this.renderer.listen(link, 'click', (event) => {
                    this.router.navigateByUrl(href);
                    event.preventDefault();
                });
            });
        });
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: RichTextDirective, deps: [{ token: i0.ViewContainerRef }, { token: i0.TemplateRef }, { token: i0.Renderer2 }, { token: i1.Router }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.13", type: RichTextDirective, selector: "[scRichText]", inputs: { editable: ["scRichTextEditable", "editable"], field: ["scRichText", "field"], emptyFieldEditingTemplate: ["scRichTextEmptyFieldEditingTemplate", "emptyFieldEditingTemplate"] }, usesInheritance: true, usesOnChanges: true, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: RichTextDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[scRichText]',
                }]
        }], ctorParameters: () => [{ type: i0.ViewContainerRef }, { type: i0.TemplateRef }, { type: i0.Renderer2 }, { type: i1.Router }], propDecorators: { editable: [{
                type: Input,
                args: ['scRichTextEditable']
            }], field: [{
                type: Input,
                args: ['scRichText']
            }], emptyFieldEditingTemplate: [{
                type: Input,
                args: ['scRichTextEmptyFieldEditingTemplate']
            }] } });

class TextDirective extends BaseFieldDirective {
    templateRef;
    editable = true;
    encode = true;
    field;
    /**
     * Custom template to render in Pages in Metadata edit mode if field value is empty
     */
    emptyFieldEditingTemplate;
    /**
     * Default component to render in Pages in Metadata edit mode if field value is empty and emptyFieldEditingTemplate is not provided
     */
    defaultFieldEditingComponent;
    constructor(viewContainer, templateRef) {
        super(viewContainer);
        this.templateRef = templateRef;
        this.defaultFieldEditingComponent = DefaultEmptyFieldEditingComponent;
    }
    ngOnChanges(changes) {
        if (changes.field || changes.editable || changes.encode) {
            this.viewContainer.clear();
            this.updateView();
        }
    }
    updateView() {
        if (!this.shouldRender()) {
            super.renderEmpty();
            return;
        }
        this.renderMetadata(MetadataKind.Open);
        this.viewRef = this.viewContainer.createEmbeddedView(this.templateRef);
        this.renderMetadata(MetadataKind.Close);
        const field = this.field;
        let editable = this.editable;
        // can't use editable value if we want to output unencoded
        if (!this.encode) {
            editable = false;
        }
        const html = field.editable && editable ? field.editable : field.value;
        const setDangerously = (field.editable && editable) || !this.encode;
        this.viewRef.rootNodes.forEach((node) => {
            if (setDangerously) {
                node.innerHTML = html;
            }
            else {
                node.textContent = html;
            }
        });
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TextDirective, deps: [{ token: i0.ViewContainerRef }, { token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.13", type: TextDirective, selector: "[scText]", inputs: { editable: ["scTextEditable", "editable"], encode: ["scTextEncode", "encode"], field: ["scText", "field"], emptyFieldEditingTemplate: ["scTextEmptyFieldEditingTemplate", "emptyFieldEditingTemplate"] }, usesInheritance: true, usesOnChanges: true, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TextDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[scText]',
                }]
        }], ctorParameters: () => [{ type: i0.ViewContainerRef }, { type: i0.TemplateRef }], propDecorators: { editable: [{
                type: Input,
                args: ['scTextEditable']
            }], encode: [{
                type: Input,
                args: ['scTextEncode']
            }], field: [{
                type: Input,
                args: ['scText']
            }], emptyFieldEditingTemplate: [{
                type: Input,
                args: ['scTextEmptyFieldEditingTemplate']
            }] } });

/**
 * @param {T} value
 * @returns {Promise | Observable | any} resolved value
 */
function wrapIntoObservable(value) {
    if (isObservable(value)) {
        return value;
    }
    if (_isPromise(value)) {
        return from(Promise.resolve(value));
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    return of(value);
}

/**
 * @param {Injector} injector
 * @param {ActivatedRoute} activatedRoute
 * @param {Router} router
 * @returns resolved data
 */
function dataResolverFactory(injector, activatedRoute, router) {
    /**
     * @param {JssResolve<unknown> | Type<JssResolve<unknown>>} resolver
     * @returns resolver instance
     */
    function getResolverInstance(resolver) {
        return 'resolve' in resolver ? resolver : injector.get(resolver);
    }
    /**
     * @param {ComponentFactoryResult} factory
     * @returns {Array<[string, JssResolve<unknown>]>} resolver instances
     */
    function collectResolverInstances(factory) {
        if (factory.resolve) {
            const resolve = factory.resolve;
            return Object.keys(factory.resolve).map((key) => [
                key,
                getResolverInstance(resolve[key]),
            ]);
        }
        return [];
    }
    /**
     * @param {JssResolve<unknown>} resolver
     * @param {ComponentFactoryResult} factory
     * @returns data
     */
    function _resolveData(resolver, factory) {
        const data = resolver.resolve({
            activatedRoute: activatedRoute.snapshot,
            routerState: router.routerState.snapshot,
            rendering: factory.componentDefinition,
            router,
        });
        const data$ = wrapIntoObservable(data);
        return lastValueFrom(data$.pipe(take(1)));
    }
    return function resolveData(factories) {
        return Promise.all(factories.map((factory) => {
            const resolvers = collectResolverInstances(factory);
            const pendingData = resolvers.map(([key, resolver]) => _resolveData(resolver, factory).then((data) => [key, data]));
            return Promise.all(pendingData)
                .then((allData) => allData.reduce((acc, [key, data]) => {
                acc[key] = data;
                return acc;
            }, {}))
                .then((data) => ({ factory, data }));
        }));
    };
}

/**
 * @param {boolean | string | string[] | UrlTree | RedirectCommand} value
 * @returns instance of value
 */
function isRedirectValue(value) {
    return (value instanceof UrlTree ||
        value instanceof RedirectCommand ||
        typeof value === 'string' ||
        Array.isArray(value));
}
/**
 * Resolves components based on provided guards
 * @param {Injector} injector
 * @param {ActivatedRoute} activatedRoute
 * @param {Router} router
 * @returns {(factories: ComponentFactoryResult[]) => Promise<ComponentFactoryResult[]>} factory
 */
function guardResolverFactory(injector, activatedRoute, router) {
    /**
     * @param {JssCanActivate | Type<JssCanActivate> | JssCanActivateFn} guard
     * @returns resolved guard
     */
    function getGuardInstance(guard) {
        if ('canActivate' in guard) {
            return guard;
        }
        try {
            // Class is provided
            return injector.get(guard);
        }
        catch {
            // Function is provided
            return guard;
        }
    }
    /**
     * @param {ComponentFactoryResult} factory
     * @returns {(JssCanActivate | JssCanActivateFn)[]} guard instances
     */
    function collectGuardInstances(factory) {
        if (factory.canActivate) {
            return Array.isArray(factory.canActivate)
                ? factory.canActivate.map(getGuardInstance)
                : [getGuardInstance(factory.canActivate)];
        }
        return [];
    }
    /**
     *
     * @param {JssCanActivate | JssCanActivateFn} guard
     * @param {ComponentFactoryResult} factory
     * @returns canActivate
     */
    function resolveGuard(guard, factory) {
        const canActivate = 'canActivate' in guard ? guard.canActivate.bind(guard) : guard;
        const guardValue = canActivate({
            activatedRoute: activatedRoute.snapshot,
            routerState: router.routerState.snapshot,
            rendering: factory.componentDefinition,
            router,
        });
        const canActivate$ = wrapIntoObservable(guardValue);
        return lastValueFrom(canActivate$.pipe(take(1), mergeMap((value) => {
            if (isRedirectValue(value)) {
                let message;
                if (value instanceof RedirectCommand) {
                    message = value.redirectTo;
                }
                else {
                    message = value;
                }
                throw new JssCanActivateRedirectError(`Value: '${message.toString()}' is a redirect value`, value);
            }
            else {
                return of(value);
            }
        })));
    }
    return function resolveGuards(factories) {
        const resolved = factories.map((factory) => {
            const guards = collectGuardInstances(factory);
            const pending = guards.map((guard) => resolveGuard(guard, factory));
            return Promise.all(pending)
                .then((canActive) => canActive.every((v) => v))
                .then((canActivate) => ({
                factory,
                canActivate,
            }));
        });
        return Promise.all(resolved).then((mapped) => mapped.filter((m) => m.canActivate).map((m) => m.factory));
    };
}

class MissingComponentComponent {
    rendering;
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: MissingComponentComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: MissingComponentComponent, selector: "sc-missing-component", inputs: { rendering: "rendering" }, ngImport: i0, template: `
    <div
      style="background: darkorange; outline: 5px solid orange; padding: 10px; color: white; max-width: 500px;"
    >
      <h2>{{ rendering.componentName }}</h2>
      <p>JSS component is missing Angular component implementation.</p>
    </div>
  `, isInline: true });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: MissingComponentComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'sc-missing-component',
                    template: `
    <div
      style="background: darkorange; outline: 5px solid orange; padding: 10px; color: white; max-width: 500px;"
    >
      <h2>{{ rendering.componentName }}</h2>
      <p>JSS component is missing Angular component implementation.</p>
    </div>
  `,
                }]
        }], propDecorators: { rendering: [{
                type: Input
            }] } });

class JssModule {
    /**
     * Instantiates the JSS module with no component factory.
     * Useful for using it from libraries. Most of the time you'd want withComponents()
     * @returns {ModuleWithProviders<JssModule>} module
     */
    static forRoot() {
        return {
            ngModule: JssModule,
            providers: [
                DatePipe,
                JssStateService,
                JssComponentFactoryService,
                {
                    provide: GUARD_RESOLVER,
                    useFactory: guardResolverFactory,
                    deps: [Injector, ActivatedRoute, Router],
                },
                {
                    provide: DATA_RESOLVER,
                    useFactory: dataResolverFactory,
                    deps: [Injector, ActivatedRoute, Router],
                },
            ],
        };
    }
    /**
     * Instantiates a module for a lazy-loaded JSS component(s)
     * @param {Type<unknown> | Record<string, Type<unknown>> } value - component or map of components
     * @returns {ModuleWithProviders<JssModule>} module
     */
    static forChild(value) {
        return {
            ngModule: JssModule,
            providers: [
                { provide: ROUTES, useValue: [], multi: true },
                { provide: DYNAMIC_COMPONENT, useValue: value },
            ],
        };
    }
    /**
     * Instantiates the JSS module and specifies the mapping from component name to component implementation.
     * Appropriate when defining the set of JSS components that your app is aware of.
     * @param {ComponentNameAndType[]} components
     * @param {ComponentNameAndModule[]} [lazyComponents]
     * @returns {ModuleWithProviders<JssModule>} module
     */
    static withComponents(components, lazyComponents) {
        return {
            ngModule: JssModule,
            providers: [
                { provide: PLACEHOLDER_COMPONENTS, useValue: components },
                { provide: PLACEHOLDER_LAZY_COMPONENTS, useValue: lazyComponents || [] },
                { provide: ROUTES, useValue: lazyComponents || [], multi: true },
                { provide: PLACEHOLDER_MISSING_COMPONENT_COMPONENT, useValue: MissingComponentComponent },
                { provide: PLACEHOLDER_HIDDEN_RENDERING_COMPONENT, useValue: HiddenRenderingComponent },
                ...JssModule.forRoot().providers,
            ],
        };
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: JssModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
    static ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "18.2.13", ngImport: i0, type: JssModule, declarations: [FileDirective,
            ImageDirective,
            LinkDirective,
            RouterLinkDirective,
            GenericLinkDirective,
            DateDirective,
            RenderEachDirective,
            RenderEmptyDirective,
            PlaceholderLoadingDirective,
            RenderComponentComponent,
            PlaceholderComponent,
            RawComponent,
            RichTextDirective,
            TextDirective,
            MissingComponentComponent,
            HiddenRenderingComponent,
            EditFrameComponent,
            EditingScriptsComponent,
            FormComponent], imports: [CommonModule], exports: [FileDirective,
            ImageDirective,
            DateDirective,
            LinkDirective,
            RouterLinkDirective,
            GenericLinkDirective,
            RenderEachDirective,
            RenderEmptyDirective,
            RenderComponentComponent,
            PlaceholderComponent,
            HiddenRenderingComponent,
            PlaceholderLoadingDirective,
            RichTextDirective,
            TextDirective,
            EditFrameComponent,
            EditingScriptsComponent,
            FormComponent] });
    static ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: JssModule, imports: [CommonModule] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: JssModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule],
                    declarations: [
                        FileDirective,
                        ImageDirective,
                        LinkDirective,
                        RouterLinkDirective,
                        GenericLinkDirective,
                        DateDirective,
                        RenderEachDirective,
                        RenderEmptyDirective,
                        PlaceholderLoadingDirective,
                        RenderComponentComponent,
                        PlaceholderComponent,
                        RawComponent,
                        RichTextDirective,
                        TextDirective,
                        MissingComponentComponent,
                        HiddenRenderingComponent,
                        EditFrameComponent,
                        EditingScriptsComponent,
                        FormComponent,
                    ],
                    exports: [
                        FileDirective,
                        ImageDirective,
                        DateDirective,
                        LinkDirective,
                        RouterLinkDirective,
                        GenericLinkDirective,
                        RenderEachDirective,
                        RenderEmptyDirective,
                        RenderComponentComponent,
                        PlaceholderComponent,
                        HiddenRenderingComponent,
                        PlaceholderLoadingDirective,
                        RichTextDirective,
                        TextDirective,
                        EditFrameComponent,
                        EditingScriptsComponent,
                        FormComponent,
                    ],
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { BaseJssState, ComponentNameAndType, DYNAMIC_COMPONENT, DateDirective, EDGE_CONFIG, EditFrameComponent, EditingScriptsComponent, FileDirective, FormComponent, GenericLinkDirective, HiddenRenderingComponent, ImageDirective, JssModule, JssStateService, LinkDirective, PlaceholderComponent, PlaceholderLoadingDirective, RenderComponentComponent, RenderEachDirective, RenderEmptyDirective, RichTextDirective, RouterLinkDirective, TextDirective, isRawRendering };
//# sourceMappingURL=sitecore-jss-sitecore-jss-angular.mjs.map
