import { InjectionToken, Type } from '@angular/core';
import { ActivatedRouteSnapshot, Data, RouterStateSnapshot, UrlTree, RedirectCommand, Router } from '@angular/router';
import { Observable } from 'rxjs';
import { ComponentFactoryResult } from './jss-component-factory.service';
import { ComponentRendering } from '../public_api';
/** Registers a statically loaded component */
export declare class ComponentNameAndType {
    name: string;
    type: Type<unknown>;
    canActivate?: JssCanActivate | Type<JssCanActivate> | JssCanActivateFn | Array<JssCanActivate | JssCanActivateFn | Type<JssCanActivate>>;
    resolve?: {
        [key: string]: JssResolve<unknown> | Type<JssResolve<unknown>>;
    };
}
/** Registers a lazily loaded component by name and module to lazy load when it's needed */
export interface ComponentNameAndModule {
    /** Name of the component */
    path: string;
    /**
     * Dynamic import of the component,
     * e.g. () => import('./path/to/lazyloadedcomponent.module').then(m => m.LazyLoadedComponentModuleExportName)
     */
    loadChildren: () => Promise<Type<unknown>>;
    canActivate?: JssCanActivate | JssCanActivateFn | Type<JssCanActivate> | Array<JssCanActivate | JssCanActivateFn | Type<JssCanActivate>>;
    resolve?: {
        [key: string]: JssResolve<unknown> | Type<JssResolve<unknown>>;
    };
}
/**
 * @param {unknown} object
 */
export declare function instanceOfComponentNameAndType(object: unknown): object is ComponentNameAndType;
/**
 * @param {unknown} object
 */
export declare function instanceOfComponentNameAndModule(object: unknown): object is ComponentNameAndModule;
export declare const PLACEHOLDER_COMPONENTS: InjectionToken<ComponentNameAndType[]>;
export declare const PLACEHOLDER_LAZY_COMPONENTS: InjectionToken<ComponentNameAndType[]>;
export declare const PLACEHOLDER_MISSING_COMPONENT_COMPONENT: InjectionToken<Type<unknown>>;
export declare const PLACEHOLDER_HIDDEN_RENDERING_COMPONENT: InjectionToken<Type<unknown>>;
export declare const DYNAMIC_COMPONENT: InjectionToken<Type<unknown> | {
    [s: string]: unknown;
}>;
export type GuardResolver = (result: ComponentFactoryResult[]) => Promise<ComponentFactoryResult[]>;
export declare const GUARD_RESOLVER: InjectionToken<GuardResolver>;
export type DataResolver = (result: ComponentFactoryResult[]) => Promise<Array<{
    factory: ComponentFactoryResult;
    data: Data;
}>>;
export declare const DATA_RESOLVER: InjectionToken<DataResolver>;
export interface GuardInput {
    activatedRoute: ActivatedRouteSnapshot;
    routerState: RouterStateSnapshot;
    rendering: ComponentRendering;
    router: Router;
}
export interface JssCanActivateFn {
    (input: GuardInput): Observable<boolean | UrlTree | RedirectCommand | string | string[]> | Promise<boolean | UrlTree | RedirectCommand | string | string[]> | boolean | UrlTree | RedirectCommand | string | string[];
}
export interface JssCanActivate {
    canActivate: JssCanActivateFn;
}
export interface JssResolve<T> {
    resolve(input: GuardInput): Observable<T> | Promise<T> | T;
}
